"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregationAutoCompleter = exports.getScopeTokensBefore = void 0;
const mongodb_constants_1 = require("@mongodb-js/mongodb-constants");
const stage_autocompleter_1 = require("./stage-autocompleter");
const util_1 = require("./util");
const STAGE_OPERATORS = mongodb_constants_1.STAGE_OPERATORS.map((op) => {
    return {
        ...op,
        snippet: `\\${op.name}: ${op.snippet}`,
    };
});
const STAGE_OPERATORS_WITH_BLOCK_SNIPPETS = STAGE_OPERATORS.map((completion) => {
    const padded = completion.snippet
        .split('\n')
        .map((line) => `  ${line}`)
        .join('\n');
    return {
        ...completion,
        snippet: `{\n${padded}\n}`,
    };
});
function type(token, type) {
    return token.type.split('.').includes(type);
}
function getPreviousTokenPos(session, row, token) {
    var _a, _b;
    if ((token === null || token === void 0 ? void 0 : token.start) === 0) {
        return {
            row: row - 1,
            column: (_a = session.$rowLengthCache[row - 1]) !== null && _a !== void 0 ? _a : session.getLine(row - 1).length,
        };
    }
    else {
        return {
            row,
            column: (_b = token === null || token === void 0 ? void 0 : token.start) !== null && _b !== void 0 ? _b : 0,
        };
    }
}
function* getScopeTokensBefore(session, pos) {
    let token = session.getTokenAt(pos.row, pos.column);
    let { row, column } = getPreviousTokenPos(session, pos.row, token);
    let skip = 0;
    while (row + column > 0) {
        token = session.getTokenAt(row, column);
        if (!token) {
            return;
        }
        skip = Math.max(0, type(token, 'paren')
            ? skip +
                (type(token, 'rparen') ? 1 : -1) *
                    token.value.length
            : skip);
        if (skip === 0) {
            yield token;
        }
        ({ row, column } = getPreviousTokenPos(session, row, token));
    }
}
exports.getScopeTokensBefore = getScopeTokensBefore;
class AggregationAutoCompleter {
    constructor(version, textCompleter, fields) {
        this.version = version;
        this.textCompleter = textCompleter;
        this.fields = fields;
        this.getCompletions = (editor, session, position, prefix, callback) => {
            const currentToken = session.getTokenAt(position.row, position.column);
            if (!currentToken) {
                return callback(null, []);
            }
            if (type(currentToken, 'comment')) {
                return callback(null, []);
            }
            const tokens = [];
            for (const token of getScopeTokensBefore(session, position)) {
                tokens.push(token);
                if (type(token, 'stage_op') ||
                    (type(token, 'identifier') &&
                        mongodb_constants_1.STAGE_OPERATOR_NAMES.includes(token.value))) {
                    this.stageAutocopmleter.updateStageOperator(token.value);
                    return this.stageAutocopmleter.getCompletions(editor, session, position, prefix, callback);
                }
            }
            callback(null, (0, util_1.filter)(this.version, isInsideBlock(tokens)
                ? STAGE_OPERATORS
                : STAGE_OPERATORS_WITH_BLOCK_SNIPPETS, prefix));
        };
        this.stageAutocopmleter = new stage_autocompleter_1.StageAutoCompleter(this.version, this.textCompleter, this.fields);
    }
    updateFields(fields) {
        this.fields = fields;
        this.stageAutocopmleter.update(fields, null);
    }
}
exports.AggregationAutoCompleter = AggregationAutoCompleter;
function isInsideBlock(tokens) {
    for (const token of tokens) {
        if (type(token, 'comment') || /^\s+$/.test(token.value)) {
            continue;
        }
        if (type(token, 'lparen') && /\{$/.test(token.value)) {
            return true;
        }
        return false;
    }
    return false;
}
//# sourceMappingURL=aggregation-autocompleter.js.map