"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongodb_constants_1 = require("@mongodb-js/mongodb-constants");
const identifierRe = '[a-zA-Z\\$_\u00a1-\uffff][a-zA-Z\\d\\$_\u00a1-\uffff]*';
function toMapperVal(items) {
    return items
        .map((item) => {
        return item.value;
    })
        .join('|');
}
ace.define('ace/mode/mongodb_highlight_rules', [
    'require',
    'exports',
    'module',
    'ace/lib/oop',
    'ace/mode/javascript_highlight_rules',
], function (acequire, exports) {
    const oop = acequire('../lib/oop');
    const { JavaScriptHighlightRules } = acequire('./javascript_highlight_rules');
    function MongoDBHighlightRules(...args) {
        oop.mixin(this, new JavaScriptHighlightRules(...args));
        const keywordMapper = this.createKeywordMapper({
            'variable.language.bson': toMapperVal(mongodb_constants_1.BSON_TYPES),
            'support.function.accumulator': toMapperVal(mongodb_constants_1.ACCUMULATORS),
            'support.function.expression_op': toMapperVal(mongodb_constants_1.EXPRESSION_OPERATORS),
            'support.function.conversion_op': toMapperVal(mongodb_constants_1.CONVERSION_OPERATORS),
            'support.function.query_op': toMapperVal(mongodb_constants_1.QUERY_OPERATORS),
            'support.function.stage_op': toMapperVal(mongodb_constants_1.STAGE_OPERATORS),
            'storage.type': 'const|let|var|function',
            'constant.language': 'null|Infinity|NaN|undefined',
            'constant.language.boolean': 'true|false',
        }, 'identifier');
        const identifierRule = this.$rules.no_regex.find((rule) => {
            return rule.regex === identifierRe;
        });
        if (!identifierRule) {
            throw new Error("Couldn't find original identifier rule. Check that the rule find operation is working correcty");
        }
        identifierRule.token = keywordMapper;
        this.normalizeRules();
    }
    oop.inherits(MongoDBHighlightRules, JavaScriptHighlightRules);
    exports.MongoDBHighlightRules = MongoDBHighlightRules;
});
ace.define('ace/mode/mongodb', [
    'require',
    'exports',
    'module',
    'ace/lib/oop',
    'ace/mode/javascript',
    'ace/mode/mongodb_highlight_rules',
], function (acequire, exports) {
    const oop = acequire('../lib/oop');
    const { Mode: JavaScriptMode } = acequire('./javascript');
    const { MongoDBHighlightRules } = acequire('./mongodb_highlight_rules');
    function MongoDBMode(...args) {
        oop.mixin(this, new JavaScriptMode(...args));
        this.HighlightRules = MongoDBHighlightRules;
        this.createWorker = () => {
            return null;
        };
    }
    oop.inherits(MongoDBMode, JavaScriptMode);
    exports.Mode = MongoDBMode;
});
//# sourceMappingURL=mode.js.map