"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StageAutoCompleter = void 0;
const semver_1 = __importDefault(require("semver"));
const mongodb_constants_1 = require("@mongodb-js/mongodb-constants");
const util_1 = require("./util");
const query_autocompleter_1 = require("./query-autocompleter");
const PROJECT = '$project';
const GROUP = '$group';
const MATCH = '$match';
const DOLLAR = '$';
const BASE_COMPLETIONS = [].concat(mongodb_constants_1.EXPRESSION_OPERATORS, mongodb_constants_1.CONVERSION_OPERATORS, mongodb_constants_1.BSON_TYPES);
class StageAutoCompleter {
    constructor(version, textCompleter, fields, stageOperator = null) {
        this.version = version;
        this.textCompleter = textCompleter;
        this.fields = fields;
        this.stageOperator = stageOperator;
        this.getCompletions = (editor, session, position, prefix, done) => {
            if (prefix === '') {
                return done(null, []);
            }
            const currentToken = session.getTokenAt(position.row, position.column);
            if (!currentToken) {
                return done(null, []);
            }
            if (currentToken.type === 'string') {
                if (prefix === DOLLAR) {
                    return done(null, this.variableFields);
                }
                return this.textCompleter.getCompletions(editor, session, position, prefix, done);
            }
            if (currentToken === null || currentToken === void 0 ? void 0 : currentToken.type.includes('comment')) {
                return done(null, []);
            }
            if (this.stageOperator && this.stageOperator === MATCH) {
                this.queryAutoCompleter.getCompletions(editor, session, position, prefix, done);
            }
            else {
                const expressions = BASE_COMPLETIONS.concat(this.accumulators()).concat(this.fields);
                return done(null, (0, util_1.filter)(this.version, expressions, prefix));
            }
        };
        this.variableFields = this.generateVariableFields(fields);
        this.queryAutoCompleter = new query_autocompleter_1.QueryAutoCompleter(version, textCompleter, fields);
    }
    accumulators() {
        if (this.stageOperator) {
            if (this.stageOperator === PROJECT) {
                return mongodb_constants_1.ACCUMULATORS.filter((acc) => {
                    if ('projectVersion' in acc) {
                        return semver_1.default.gte(this.version, acc.projectVersion);
                    }
                    return false;
                });
            }
            else if (this.stageOperator === GROUP) {
                return mongodb_constants_1.ACCUMULATORS;
            }
        }
        return [];
    }
    update(fields, stageOperator, severVersion) {
        this.fields = fields;
        this.variableFields = this.generateVariableFields(fields);
        this.queryAutoCompleter.update(fields);
        this.stageOperator = stageOperator;
        this.version = severVersion !== null && severVersion !== void 0 ? severVersion : this.version;
    }
    updateStageOperator(stageOperator) {
        this.stageOperator = stageOperator;
    }
    generateVariableFields(fields) {
        return fields.map((field) => {
            return {
                ...(field.name && { name: `$${field.name.replace(/"/g, '')}` }),
                value: `$${field.value.replace(/"/g, '')}`,
                meta: field.meta,
                version: field.version,
                score: 1,
            };
        });
    }
}
exports.StageAutoCompleter = StageAutoCompleter;
//# sourceMappingURL=stage-autocompleter.js.map