"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setEditorValue = exports.EditorTextCompleter = exports.EditorVariant = exports.BaseEditor = void 0;
const react_1 = __importStar(require("react"));
const compass_components_1 = require("@mongodb-js/compass-components");
const compass_components_2 = require("@mongodb-js/compass-components");
require("ace-builds");
const react_ace_1 = __importDefault(require("react-ace"));
require("ace-builds/src-noconflict/ext-language_tools");
require("ace-builds/src-noconflict/mode-javascript");
require("ace-builds/src-noconflict/mode-json");
require("ace-builds/src-noconflict/mode-csharp");
require("ace-builds/src-noconflict/mode-python");
require("ace-builds/src-noconflict/mode-java");
require("ace-builds/src-noconflict/mode-ruby");
require("ace-builds/src-noconflict/mode-rust");
require("ace-builds/src-noconflict/mode-golang");
require("ace-builds/src-noconflict/mode-php");
const ext_language_tools_1 = __importDefault(require("ace-builds/src-noconflict/ext-language_tools"));
require("./ace");
const prettify_1 = require("./prettify");
const DEFAULT_OPTIONS = {
    enableLiveAutocompletion: false,
    tabSize: 2,
    minLines: 10,
    maxLines: Infinity,
    showGutter: true,
    showPrintMargin: false,
    useWorker: false,
    fontFamily: compass_components_2.fontFamilies.code,
    fontSize: 13,
};
const EditorVariant = {
    Shell: 'Shell',
    EJSON: 'EJSON',
    Generic: 'Generic',
};
exports.EditorVariant = EditorVariant;
const editorStyle = (0, compass_components_2.css)({
    position: 'relative',
    width: '100%',
    zIndex: 0,
});
const defaultCommands = [
    {
        name: 'prettify',
        exec(editor) {
            try {
                const mode = editor.session.getMode().$id;
                const format = mode.endsWith('/json')
                    ? 'json'
                    : 'javascript-expression';
                const code = (0, prettify_1.prettify)(editor.getValue(), format);
                const currentSelection = editor.selection.toJSON();
                editor.session.doc.setValue(code);
                editor.selection.fromJSON(currentSelection);
            }
            catch (_a) {
            }
        },
        bindKey: {
            win: 'Ctrl-Shift-B',
            mac: 'Ctrl-Shift-B',
        },
    },
    {
        name: 'copy-all',
        exec(editor) {
            void navigator.clipboard.writeText(editor.getValue());
        },
        bindKey: {
            win: 'Ctrl-Shift-C',
            mac: 'Ctrl-Shift-C',
        },
    },
];
function BaseEditor({ text, variant, options, readOnly, id, name, commands: _commands, onChangeText, completer, 'data-testid': dataTestId, onFocus, darkMode: _darkMode, className, editorClassName, ...aceProps }) {
    const setOptions = {
        ...DEFAULT_OPTIONS,
        ...options,
        ...(typeof readOnly === 'boolean' && { readOnly }),
        ...(variant === 'Shell' && { mode: 'ace/mode/mongodb' }),
        ...(!!completer && { enableLiveAutocompletion: true }),
    };
    const editorRef = (0, react_1.useRef)(null);
    const providerDarkMode = (0, compass_components_2.useDarkMode)();
    const darkMode = _darkMode !== null && _darkMode !== void 0 ? _darkMode : providerDarkMode;
    (0, react_1.useLayoutEffect)(() => {
        if (!editorRef.current) {
            return;
        }
        editorRef.current.editor.setTheme(`ace/theme/${darkMode ? 'mongodb-dark' : 'mongodb'}`);
    }, [darkMode]);
    (0, react_1.useLayoutEffect)(() => {
        if (id && editorRef.current) {
            editorRef.current.editor.textInput.getElement().id = id;
        }
    }, [id]);
    const editorName = (0, compass_components_2.useId)();
    const commands = (0, react_1.useMemo)(() => {
        return defaultCommands.concat(_commands !== null && _commands !== void 0 ? _commands : []);
    }, [_commands]);
    return (react_1.default.createElement("div", { "data-testid": dataTestId, className: (0, compass_components_1.cx)(editorStyle, className) },
        react_1.default.createElement(react_ace_1.default, { ref: editorRef, mode: variant === 'Generic'
                ? undefined
                : variant === 'EJSON'
                    ? 'json'
                    : 'javascript', width: "100%", value: text, onChange: onChangeText, editorProps: { $blockScrolling: Infinity }, setOptions: setOptions, readOnly: readOnly, name: name !== null && name !== void 0 ? name : editorName, commands: commands, onFocus: (ev) => {
                if (completer) {
                    ext_language_tools_1.default.setCompleters([completer]);
                }
                onFocus === null || onFocus === void 0 ? void 0 : onFocus(ev);
            }, className: editorClassName, ...aceProps })));
}
exports.BaseEditor = BaseEditor;
function setEditorValue(element, value) {
    const container = element.querySelector('.ace_editor');
    if (!container) {
        throw new Error('Cannot find editor container');
    }
    window.ace
        .edit(element.querySelector(`#${container.id}`))
        .setValue(value);
}
exports.setEditorValue = setEditorValue;
const EditorTextCompleter = ext_language_tools_1.default.textCompleter;
exports.EditorTextCompleter = EditorTextCompleter;
//# sourceMappingURL=base-editor.js.map